@@ -338,6 +338,7 @@ RESI_NAMES: List[str] = [
     "LYS",
     "MET",
     "MLE",
+    "MSE",
     "MVA",
     "NH2",
     "NLE",
@@ -434,6 +435,7 @@ RESI_THREE_TO_1: Dict[str, str] = {
     "LYS": "K",
     "MET": "M",
     "MLE": "L",
+    "MSE": "M",
     "MVA": "V",
     "NH2": "X",
     "NLE": "L",
