@@ -7,6 +7,7 @@
 from __future__ import annotations
 
 import logging
+import re
 from itertools import count
 from typing import Dict, List, Optional, Tuple, Union
 
@@ -14,6 +15,7 @@ import matplotlib
 import matplotlib.pyplot as plt
 import networkx as nx
 import numpy as np
+import plotly.colors as co
 import plotly.express as px
 import plotly.graph_objects as go
 import seaborn as sns
@@ -93,7 +95,7 @@ def colour_nodes(
     # Define color range proportional to number of edges adjacent to a single node
     if colour_by == "degree":
         # Get max number of edges connected to a single node
-        edge_max = max([G.degree[i] for i in G.nodes()])
+        edge_max = max(G.degree[i] for i in G.nodes())
         colors = [colour_map(G.degree[i] / edge_max) for i in G.nodes()]
     elif colour_by == "seq_position":
         colors = [colour_map(i / n) for i in range(n)]
@@ -103,6 +105,20 @@ def colour_nodes(
             zip(chains, list(colour_map(1 / len(chains), 1, len(chains))))
         )
         colors = [chain_colours[d["chain_id"]] for n, d in G.nodes(data=True)]
+    elif colour_by == "plddt":
+        levels: List[str] = ["Very High", "Confident", "Low", "Very Low"]
+        mapping = dict(zip(sorted(levels), count()))
+        colors = []
+        for _, d in G.nodes(data=True):
+            if d["b_factor"] > 90:
+                colors.append((27 / 256, 86 / 256, 206 / 256, 1))
+            elif d["b_factor"] > 70:
+                colors.append((126 / 256, 202 / 256, 239 / 256, 1))
+            elif d["b_factor"] > 50:
+                colors.append((250 / 256, 218 / 256, 77 / 256, 1))
+            else:
+                colors.append((239 / 256, 131 / 256, 83 / 256, 1))
+        # colors = [colour_map(mapping[c] / len(levels)) for c in colors]
     else:
         node_types = set(nx.get_node_attributes(G, colour_by).values())
         mapping = dict(zip(sorted(node_types), count()))
@@ -117,8 +133,10 @@ def colour_nodes(
 def colour_edges(
     G: nx.Graph,
     colour_map: matplotlib.colors.ListedColormap,
-    colour_by: str = "kind",
-) -> List[Tuple[float, float, float, float]]:
+    colour_by: Optional[str] = "kind",
+    set_alpha: float = 1.0,
+    return_as_rgba: bool = False,
+) -> List[Tuple[float, float, float, float]] or List[str]:
     """
     Computes edge colours based on the kind of bond/interaction.
 
@@ -127,14 +145,18 @@ def colour_edges(
     :param colour_map: Colourmap to use.
     :type colour_map: matplotlib.colors.ListedColormap
     :param colour_by: Edge attribute to colour by. Currently only ``"kind"`` is supported.
-    :type colour_by: str
+    :type colour_by: Optional[str]
+    :param set_alpha: Sets a given alpha value between 0.0 and 1.0 for all the edge colours.
+    :type set_alpha: float
+    :param return_as_rgba: Returns a list of rgba strings instead of tuples.
+    :type return_as_rgba: bool
     :return: List of edge colours.
-    :rtype: List[Tuple[float, float, float, float]]
+    :rtype: List[Tuple[float, float, float, float]] or List[str]
     """
     if colour_by == "kind":
-        edge_types = set(
+        edge_types = {
             frozenset(a) for a in nx.get_edge_attributes(G, "kind").values()
-        )
+        }
         mapping = dict(zip(sorted(edge_types), count()))
         colors = [
             colour_map(
@@ -143,10 +165,25 @@ def colour_edges(
             )
             for i in G.edges()
         ]
+    elif colour_by is None:
+        colors = [(0.0, 0.0, 0.0, 1.0) for _ in G.edges()]
     else:
-        raise NotImplementedError(
-            "Other edge colouring methods not implemented."
-        )
+        edge_types = set(nx.get_edge_attributes(G, colour_by).values())
+        mapping = dict(zip(sorted(edge_types), count()))
+        colors = [
+            colour_map(mapping[d[colour_by]] / len(edge_types))
+            for _, _, d in G.edges(data=True)
+        ]
+
+    assert (
+        0.0 <= set_alpha <= 1.0
+    ), f"Alpha value {set_alpha} must be between 0.0 and 1.0"
+    colors = [c[:3] + (set_alpha,) for c in colors]
+    if return_as_rgba:
+        return [
+            f"rgba{tuple(list(co.convert_to_RGB_255(c[:3])) + [c[3]])}"
+            for c in colors
+        ]
     return colors
 
 
@@ -157,11 +194,12 @@ def plotly_protein_structure_graph(
     node_alpha: float = 0.7,
     node_size_min: float = 20.0,
     node_size_multiplier: float = 20.0,
+    node_size_feature: str = "degree",
     label_node_ids: bool = True,
     node_colour_map=plt.cm.plasma,
     edge_color_map=plt.cm.plasma,
     colour_nodes_by: str = "degree",
-    colour_edges_by: str = "kind",
+    colour_edges_by: Optional[str] = "kind",
 ) -> go.Figure:
     """
     Plots protein structure graph using plotly.
@@ -178,6 +216,8 @@ def plotly_protein_structure_graph(
     :type node_size_min: float
     :param node_size_multiplier: Scales node size by a constant. Node sizes reflect degree. Defaults to ``20.0``.
     :type node_size_multiplier: float
+    :param node_size_feature: Which feature to scale the node size by. Defaults to ``degree``.
+    :type node_size_feature: str
     :param label_node_ids: bool indicating whether or not to plot ``node_id`` labels. Defaults to ``True``.
     :type label_node_ids: bool
     :param node_colour_map: colour map to use for nodes. Defaults to ``plt.cm.plasma``.
@@ -185,9 +225,9 @@ def plotly_protein_structure_graph(
     :param edge_color_map: colour map to use for edges. Defaults to ``plt.cm.plasma``.
     :type edge_color_map: plt.cm
     :param colour_nodes_by: Specifies how to colour nodes. ``"degree"``, ``"seq_position"`` or a node feature.
-    :type colour_edges_by: str
-    :param colour_edges_by: Specifies how to colour edges. Currently only ``"kind"`` is supported.
     :type colour_nodes_by: str
+    :param colour_edges_by: Specifies how to colour edges. Currently only ``"kind"`` or ``None`` are supported.
+    :type colour_edges_by: Optional[str]
     :returns: Plotly Graph Objects plot
     :rtype: go.Figure
     """
@@ -203,6 +243,28 @@ def plotly_protein_structure_graph(
         G, colour_map=edge_color_map, colour_by=colour_edges_by
     )
 
+    # Get node size
+    def node_scale_by(G: nx.Graph, feature: str):
+        if feature == "degree":
+            return lambda k: node_size_min + node_size_multiplier * G.degree[k]
+        elif feature == "rsa":
+            return (
+                lambda k: node_size_min
+                + node_size_multiplier * G.nodes(data=True)[k]["rsa"]
+            )
+
+        # Meiler embedding dimension
+        p = re.compile("meiler-([1-7])")
+        dim = p.search(feature).group(1)
+        if dim:
+            return lambda k: node_size_min + node_size_multiplier * max(
+                0, G.nodes(data=True)[k]["meiler"][f"dim_{dim}"]
+            )  # Meiler values may be negative
+        else:
+            raise ValueError(f"Cannot size nodes by feature '{feature}'")
+
+    get_node_size = node_scale_by(G, node_size_feature)
+
     # 3D network plot
     x_nodes = []
     y_nodes = []
@@ -215,7 +277,7 @@ def plotly_protein_structure_graph(
         x_nodes.append(value[0])
         y_nodes.append(value[1])
         z_nodes.append(value[2])
-        node_sizes.append(node_size_min + node_size_multiplier * G.degree[key])
+        node_sizes.append(get_node_size(key))
 
         if label_node_ids:
             node_labels.append(list(G.nodes())[i])
@@ -654,12 +716,14 @@ def asteroid_plot(
     colour_nodes_by: str = "shell",  # residue_name
     colour_edges_by: str = "kind",
     edge_colour_map: plt.cm.Colormap = plt.cm.plasma,
+    edge_alpha: float = 1.0,
     show_labels: bool = True,
     title: Optional[str] = None,
     width: int = 600,
     height: int = 500,
     use_plotly: bool = True,
     show_edges: bool = False,
+    show_legend: bool = True,
     node_size_multiplier: float = 10,
 ) -> Union[plotly.graph_objects.Figure, matplotlib.figure.Figure]:
     """Plots a k-hop subgraph around a node as concentric shells.
@@ -678,6 +742,8 @@ def asteroid_plot(
     :type colour_edges_by: str
     :param edge_colour_map: Colour map for edges. Defaults to ``plt.cm.plasma``.
     :type edge_colour_map: plt.cm.Colormap
+    :param edge_alpha: Sets a given alpha value between 0.0 and 1.0 for all the edge colours.
+    :type edge_alpha: float
     :param title: Title of the plot. Defaults to ``None``.
     :type title: str
     :param width: Width of the plot. Defaults to ``600``.
@@ -686,8 +752,10 @@ def asteroid_plot(
     :type height: int
     :param use_plotly: Use plotly to render the graph. Defaults to ``True``.
     :type use_plotly: bool
-    :param show_edges: Whether or not to show edges in the plot. Defaults to ``False``.
+    :param show_edges: Whether to show edges in the plot. Defaults to ``False``.
     :type show_edges: bool
+    :param show_legend: Whether to show the legend of the edges. Fefaults to `True``.
+    :type show_legend: bool
     :param node_size_multiplier: Multiplier for the size of the nodes. Defaults to ``10``.
     :type node_size_multiplier: float.
     :returns: Plotly figure or matplotlib figure.
@@ -695,8 +763,7 @@ def asteroid_plot(
     """
     assert node_id in g.nodes(), f"Node {node_id} not in graph"
 
-    nodes: Dict[int, List[str]] = {}
-    nodes[0] = [node_id]
+    nodes: Dict[int, List[str]] = {0: [node_id]}
     node_list: List[str] = [node_id]
     # Iterate over the number of hops and extract nodes in each shell
     for i in range(1, k):
@@ -715,34 +782,33 @@ def asteroid_plot(
 
         if show_edges:
             edge_colors = colour_edges(
-                subgraph, colour_map=edge_colour_map, colour_by=colour_edges_by
+                subgraph,
+                colour_map=edge_colour_map,
+                colour_by=colour_edges_by,
+                set_alpha=edge_alpha,
+                return_as_rgba=True,
             )
-
-            edge_x: List[str] = []
-            edge_y: List[str] = []
-            edge_type: List[str] = []
-            for u, v in subgraph.edges():
+            show_legend_bools = [
+                (True if x not in edge_colors[:i] else False)
+                for i, x in enumerate(edge_colors)
+            ]
+            edge_trace = []
+            for i, (u, v) in enumerate(subgraph.edges()):
                 x0, y0 = subgraph.nodes[u]["pos"]
                 x1, y1 = subgraph.nodes[v]["pos"]
-                edge_x.append(x0)
-                edge_x.append(x1)
-                edge_x.append(None)
-                edge_y.append(y0)
-                edge_y.append(y1)
-                edge_y.append(None)
-            edge_trace = go.Scatter(
-                x=edge_x,
-                y=edge_y,
-                line=dict(width=1, color=edge_colors),
-                hoverinfo="text",
-                mode="lines",
-                text=[
-                    " / ".join(list(edge_type))
-                    for edge_type in nx.get_edge_attributes(
-                        subgraph, "kind"
-                    ).values()
-                ],
-            )
+                bond_kind = " / ".join(list(subgraph[u][v]["kind"]))
+                tr = go.Scatter(
+                    x=(x0, x1),
+                    y=(y0, y1),
+                    mode="lines",
+                    line=dict(width=1, color=edge_colors[i]),
+                    hoverinfo="text",
+                    text=[bond_kind],
+                    name=bond_kind,
+                    legendgroup=bond_kind,
+                    showlegend=show_legend_bools[i],
+                )
+                edge_trace.append(tr)
 
         node_x: List[str] = []
         node_y: List[str] = []
@@ -773,6 +839,7 @@ def asteroid_plot(
             mode="markers+text" if show_labels else "markers",
             hoverinfo="text",
             textposition="bottom center",
+            showlegend=False,
             marker=dict(
                 colorscale="YlGnBu",
                 reversescale=True,
@@ -789,7 +856,7 @@ def asteroid_plot(
             ),
         )
 
-        data = [edge_trace, node_trace] if show_edges else [node_trace]
+        data = edge_trace + [node_trace] if show_edges else [node_trace]
         fig = go.Figure(
             data=data,
             layout=go.Layout(
@@ -797,7 +864,8 @@ def asteroid_plot(
                 width=width,
                 height=height,
                 titlefont_size=16,
-                showlegend=False,
+                legend=dict(yanchor="top", y=1, xanchor="left", x=1.10),
+                showlegend=True if show_legend else False,
                 hovermode="closest",
                 margin=dict(b=20, l=5, r=5, t=40),
                 xaxis=dict(
@@ -925,55 +993,3 @@ def plot_chord_diagram(
         show=show,
         **kwargs,
     )
-
-
-if __name__ == "__main__":
-    # TODO: Move the block here into tests.
-    from graphein.protein.config import ProteinGraphConfig
-    from graphein.protein.edges.atomic import (
-        add_atomic_edges,
-        add_bond_order,
-        add_ring_status,
-    )
-    from graphein.protein.features.nodes.amino_acid import (
-        expasy_protein_scale,
-        meiler_embedding,
-    )
-    from graphein.protein.graphs import construct_graph
-
-    # Test Point cloud plotting
-    # v, f, a = create_mesh(pdb_code="3eiy")
-    # m = convert_verts_and_face_to_mesh(v, f)
-    # plot_pointcloud(m, "Test")
-    # TEST PROTEIN STRUCTURE GRAPH PLOTTING
-    configs = {
-        "granularity": "atom",
-        "keep_hets": False,
-        "deprotonate": True,
-        "insertions": False,
-        "verbose": False,
-    }
-
-    config = ProteinGraphConfig(**configs)
-    config.edge_construction_functions = [
-        add_atomic_edges,
-        add_ring_status,
-        add_bond_order,
-    ]
-
-    config.node_metadata_functions = [meiler_embedding, expasy_protein_scale]
-
-    g = construct_graph(
-        config=config, pdb_path="../examples/pdbs/3eiy.pdb", pdb_code="3eiy"
-    )
-
-    p = plotly_protein_structure_graph(
-        g,
-        30,
-        (1000, 2000),
-        colour_nodes_by="element_symbol",
-        colour_edges_by="kind",
-        label_node_ids=False,
-    )
-
-    p.show()
